"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const objection_1 = require("objection");
exports.Model = objection_1.Model;
exports.QueryBuilder = objection_1.QueryBuilder;
exports.knex = require("knex");
function Table(options) {
    return function (target) {
        target.tableName = target.name;
        Object.keys(options).forEach(item => {
            target[item] = options[item];
        });
    };
}
exports.Table = Table;
const allTypes = {
    number: 'number',
    string: 'string',
    boolean: 'boolean',
    object: 'object',
    array: 'array',
    null: 'null',
};
const anyType = Object
    .values(allTypes)
    .map(item => item);
exports.columnTypes = {
    increments: { columnType: 'increments', type: [allTypes.number, allTypes.null] },
    bigIncrements: { columnType: 'bigIncrements', type: [allTypes.number, allTypes.null] },
    integer: { columnType: 'integer', type: [allTypes.number, allTypes.null] },
    bigInteger: { columnType: 'bigInteger', type: [allTypes.number, allTypes.null] },
    text: { columnType: 'text', type: [allTypes.string, allTypes.null] },
    string: { columnType: 'string', type: [allTypes.string, allTypes.null] },
    float: { columnType: 'float', type: anyType },
    decimal: { columnType: 'decimal', type: anyType },
    boolean: { columnType: 'boolean', type: [allTypes.boolean] },
    date: { columnType: 'date', type: anyType },
    datetime: { columnType: 'datetime', type: anyType },
    time: { columnType: 'time', type: anyType },
    timestamp: { columnType: 'timestamp', type: anyType },
    timestamps: { columnType: 'timestamps', type: anyType },
    binary: { columnType: 'binary', type: anyType },
    json: { columnType: 'json', type: [allTypes.object] },
    jsonb: { columnType: 'jsonb', type: [allTypes.object] },
    uuid: { columnType: 'uuid', type: anyType },
    number: { columnType: 'integer', type: [allTypes.number, allTypes.null] },
    object: { columnType: 'json', type: [allTypes.object] }
};
function Column(options) {
    return function (target, propertyKey) {
        var _a, _b;
        const columnName = options.columnName || propertyKey;
        if (((_a = options === null || options === void 0 ? void 0 : options.type) === null || _a === void 0 ? void 0 : _a.columnType) === 'increments' || ((_b = options === null || options === void 0 ? void 0 : options.type) === null || _b === void 0 ? void 0 : _b.columnType) === 'bigIncrements' || (options === null || options === void 0 ? void 0 : options.primary)) {
            target.constructor.idColumn = columnName;
        }
        target.constructor._columnOptions = target.constructor._columnOptions || {};
        target.constructor._columnOptions[propertyKey] = Object.assign(Object.assign({}, options), { columnName }) || {};
        if (options && options.type) {
            target.constructor.jsonSchema = target.constructor.jsonSchema || {};
            target.constructor.jsonSchema.properties = target.constructor.jsonSchema.properties || {};
            target.constructor.jsonSchema.properties[propertyKey] = options.type;
        }
    };
}
exports.Column = Column;
exports.relationTypes = {
    HasManyRelation: objection_1.Model.HasManyRelation,
    HasOneRelation: objection_1.Model.HasOneRelation,
    HasOneThroughRelation: objection_1.Model.HasOneThroughRelation,
    BelongsToOneRelation: objection_1.Model.BelongsToOneRelation,
};
function Relation(relationMapping) {
    return function (target, propertyKey) {
        target.constructor.relationMappings = target.constructor.relationMappings || {};
        target.constructor.relationMappings[propertyKey] = relationMapping;
    };
}
exports.Relation = Relation;
async function synchronize(model, force) {
    const tableName = model.tableName;
    if (force && await model.knex().schema.hasTable(tableName)) {
        await model.knex().schema.dropTable(tableName);
    }
    const properties = model['_columnOptions'];
    if (!await model.knex().schema.hasTable(tableName)) {
        await model.knex().schema.createTable(tableName, table => {
            const increments = Object.values(properties).filter(item => { var _a; return ((_a = item.type) === null || _a === void 0 ? void 0 : _a.columnType) === 'increments'; })[0];
            const bigIncrements = Object.values(properties).filter(item => { var _a; return ((_a = item.type) === null || _a === void 0 ? void 0 : _a.columnType) === 'bigIncrements'; })[0];
            const primary = Object.values(properties).filter(item => item.primary)[0];
            if (increments) {
                table.increments(increments.columnName).primary();
            }
            else if (bigIncrements) {
                table.bigIncrements(bigIncrements.columnName).primary();
            }
            else if (primary) {
                table[primary.type.columnType](primary.columnName);
            }
            else {
                table.increments('id').primary();
            }
        });
    }
    for (const item of Object.keys(properties)) {
        const _a = properties[item], { type: { columnType }, columnName } = _a, options = __rest(_a, ["type", "columnName"]);
        if (!await model.knex().schema.hasColumn(tableName, columnName)) {
            await model.knex().schema.table(tableName, table => {
                var _a, _b;
                table[columnType](columnName);
                const client = (_b = (_a = model.knex().client) === null || _a === void 0 ? void 0 : _a.config) === null || _b === void 0 ? void 0 : _b.client;
                if (client === 'pg' || client === 'mysql' || client === 'mysql2') {
                    if (options.default) {
                        table[columnType](columnName).default(options.default).alter();
                    }
                    if (options.unique) {
                        table[columnType](columnName).unique().alter();
                    }
                    if (options.unsigned) {
                        table[columnType](columnName).unsigned().alter();
                    }
                    if (options.nullable) {
                        table[columnType](columnName).nullable().alter();
                    }
                    if (options.notNullable) {
                        table[columnType](columnName).notNullable().alter();
                    }
                }
            });
        }
    }
}
exports.synchronize = synchronize;
