"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const objection_1 = require("objection");
exports.softDelete = () => {
    return (IncomingModel) => {
        class SoftDeleteQueryBuilder extends IncomingModel.QueryBuilder {
            constructor(modelClass) {
                super(modelClass);
                if (modelClass.softDelete) {
                    this.softDeleteColumnName = typeof modelClass.softDelete === 'string' ? modelClass.softDelete : 'deletedAt';
                    this.onBuild((q) => {
                        if (q.isFind() && !q.context().includeDeleted) {
                            q.whereNull(`${q.tableRefFor(modelClass)}.${this.softDeleteColumnName}`);
                        }
                    });
                }
            }
            delete(...args) {
                if (this.modelClass().softDelete) {
                    return super.patch({ [this.softDeleteColumnName]: new Date() });
                }
                return super.delete(...args);
            }
            forceDelete(...args) {
                return super.delete(...args);
            }
            withDeleted() {
                return this.context({ includeDeleted: true });
            }
            onlyDeleted() {
                return this.withDeleted().whereNotNull(this.softDeleteColumnName);
            }
            softDelete() {
                if (this.modelClass().softDelete) {
                    return super.patch({ [this.softDeleteColumnName]: new Date() });
                }
            }
            restore() {
                if (this.modelClass().softDelete) {
                    return super.patch({ [this.softDeleteColumnName]: null });
                }
            }
        }
        class SDModel extends IncomingModel {
            static get QueryBuilder() {
                return SoftDeleteQueryBuilder;
            }
            static get isSoftDelete() {
                return true;
            }
        }
        ;
        return SDModel;
    };
};
class SoftDeleteModel extends exports.softDelete()(objection_1.Model) {
}
exports.SoftDeleteModel = SoftDeleteModel;
;
